@@ -23,10 +23,45 @@
 
 #include "abstractmetalang.h"
 #include "reporthandler.h"
+#include "typedatabase.h"
+
+/*******************************************************************************
+ * AbstractMetaVariable
+ */
+
+AbstractMetaVariable::AbstractMetaVariable(const AbstractMetaVariable &other)
+{
+    m_originalName = other.m_originalName;
+    m_name = other.m_name;
+    m_type = other.m_type->copy();
+    m_hasName = other.m_hasName;
+    m_doc = other.m_doc;
+}
 
 /*******************************************************************************
  * AbstractMetaType
  */
+
+AbstractMetaType::AbstractMetaType()
+    :m_typeEntry(0),
+    m_arrayElementCount(0),
+    m_arrayElementType(0),
+    m_originalTemplateType(0),
+    m_pattern(InvalidPattern),
+    m_constant(false),
+    m_reference(false),
+    m_cppInstantiation(true),
+    m_indirections(0),
+    m_reserved(0)
+{
+}
+
+AbstractMetaType::~AbstractMetaType()
+{
+    qDeleteAll(m_children);
+    m_instantiations.clear();
+}
+
 AbstractMetaType *AbstractMetaType::copy() const
 {
     AbstractMetaType *cpy = new AbstractMetaType;
@@ -57,12 +92,12 @@ QString AbstractMetaType::cppSignature() const
     s += typeEntry()->qualifiedCppName();
 
     if (hasInstantiationInCpp()) {
-        QList<AbstractMetaType *> types = instantiations();
+        AbstractMetaTypeList types = instantiations();
         s += "<";
         for (int i = 0; i < types.count(); ++i) {
             if (i > 0)
                 s += ", ";
-            s += types.at(i)->cppSignature();
+            s += types[i]->cppSignature();
         }
         s += " >";
     }
@@ -77,46 +112,15 @@ QString AbstractMetaType::cppSignature() const
     return s;
 }
 
+
 /*******************************************************************************
  * AbstractMetaArgument
  */
 AbstractMetaArgument *AbstractMetaArgument::copy() const
 {
-    AbstractMetaArgument *cpy = new AbstractMetaArgument;
-    cpy->setName(AbstractMetaVariable::name());
-    cpy->setDefaultValueExpression(defaultValueExpression());
-    cpy->setOriginalDefaultValueExpression(originalDefaultValueExpression());
-    cpy->setType(type()->copy());
-    cpy->setArgumentIndex(argumentIndex());
-
-    return cpy;
+    return new AbstractMetaArgument(*this);
 }
 
-
-QString AbstractMetaArgument::argumentName() const
-{
-    QString n = AbstractMetaVariable::name();
-    if (n.isEmpty())
-        return QString("arg__%2").arg(m_argumentIndex + 1);
-    return n;
-}
-
-
-QString AbstractMetaArgument::indexedName() const
-{
-    QString n = AbstractMetaVariable::name();
-    if (n.isEmpty())
-        return argumentName();
-    return QString("%1%2").arg(n).arg(m_argumentIndex);
-}
-
-QString AbstractMetaArgument::name() const
-{
-    Q_ASSERT_X(0, "AbstractMetaArgument::name()", "use argumentName() or indexedName() instead");
-    return QString();
-}
-
-
 /*******************************************************************************
  * AbstractMetaFunction
  */
@@ -334,7 +338,7 @@ QString AbstractMetaFunction::signature() const
 
         // We need to have the argument names in the qdoc files
         s += " ";
-        s += a->argumentName();
+        s += a->name();
     }
     s += ")";
 
@@ -686,8 +690,9 @@ FunctionModificationList AbstractMetaFunction::modifications(const AbstractMetaC
     FunctionModificationList mods;
     while (implementor) {
         mods += implementor->typeEntry()->functionModifications(minimalSignature());
-        if (implementor == implementor->baseClass() || implementor == implementingClass())
-            break;
+        if ((implementor == implementor->baseClass()) ||
+            (implementor == implementingClass() && (mods.size() > 0)))
+                break;
         implementor = implementor->baseClass();
     }
     return mods;
@@ -698,6 +703,11 @@ bool AbstractMetaFunction::hasModifications(const AbstractMetaClass *implementor
     return !modifications(implementor).isEmpty();
 }
 
+QString AbstractMetaFunction::argumentName(int index, bool create, const AbstractMetaClass *implementor) const
+{
+    return m_arguments[--index]->name();
+}
+
 bool AbstractMetaFunction::hasInjectedCode() const
 {
     foreach (const FunctionModification mod, modifications(ownerClass())) {
@@ -711,7 +721,7 @@ CodeSnipList AbstractMetaFunction::injectedCodeSnips(CodeSnip::Position position
 {
     CodeSnipList result;
     foreach (const FunctionModification mod, modifications(ownerClass())) {
-        if (mod.isCodeInjection()) {
+        if (mod.isCodeInjection() && TypeDatabase::instance()->supportedApiVersion(mod.version())) {
             QList<CodeSnip>::const_iterator it = mod.snips.constBegin();
             for (;it != mod.snips.constEnd(); ++it) {
                 if ((it->language & language) && (it->position == position || position == CodeSnip::Any))
@@ -752,6 +762,7 @@ bool AbstractMetaFunction::isOperatorOverload(QString funcName)
     QRegExp opRegEx("^operator([+\\-\\*/%=&\\|\\^\\<>!][=]?"
                     "|\\+\\+|\\-\\-|&&|\\|\\||<<[=]?|>>[=]?|~"
                     "|\\[\\]|\\s+delete\\[?\\]?"
+                    "|\\(\\)"
                     "|\\s+new\\[?\\]?)$");
     return opRegEx.indexIn(funcName) > -1;
 }
@@ -931,7 +942,7 @@ QString AbstractMetaFunction::targetLangSignature(bool minimal) const
 
         if (!minimal) {
             s += " ";
-            s += m_arguments.at(i)->argumentName();
+            s += m_arguments.at(i)->name();
         }
         ++j;
     }
@@ -954,6 +965,8 @@ AbstractMetaClass::~AbstractMetaClass()
 {
     qDeleteAll(m_functions);
     qDeleteAll(m_fields);
+    qDeleteAll(m_enums);
+    qDeleteAll(m_orphanInterfaces);
 }
 
 /*******************************************************************************
@@ -1008,6 +1021,7 @@ AbstractMetaClass *AbstractMetaClass::extractInterface()
 
         m_extractedInterface = iface;
         addInterface(iface);
+        m_orphanInterfaces << iface;
     }
 
     return m_extractedInterface;
@@ -1102,18 +1116,28 @@ AbstractMetaFunctionList AbstractMetaClass::nonVirtualShellFunctions() const
 
 AbstractMetaFunctionList AbstractMetaClass::implicitConversions() const
 {
-    AbstractMetaFunctionList list = queryFunctions(Constructors);
+    if (!hasCloneOperator() && !hasExternalConversionOperators())
+        return AbstractMetaFunctionList();
+
     AbstractMetaFunctionList returned;
+    AbstractMetaFunctionList list = queryFunctions(Constructors);
+    list.append(externalConversionOperators());
+
     foreach (AbstractMetaFunction *f, list) {
-        if (f->arguments().size() == 1 && !f->isExplicit() && !f->isCopyConstructor() && hasCloneOperator())
+        if ((f->actualMinimumArgumentCount() == 1 || f->arguments().size() == 1 || f->isConversionOperator())
+            && !f->isExplicit()
+            && !f->isCopyConstructor()
+            && !f->isModifiedRemoved()
+            && (f->originalAttributes() & Public)) {
             returned += f;
+        }
     }
     return returned;
 }
 
 AbstractMetaFunctionList AbstractMetaClass::operatorOverloads(uint query) const
 {
-    AbstractMetaFunctionList list = queryFunctions(OperatorOverloads);
+    AbstractMetaFunctionList list = queryFunctions(OperatorOverloads | Visible);
     AbstractMetaFunctionList returned;
     foreach (AbstractMetaFunction *f, list) {
         if (((query & ArithmeticOp) && f->isArithmeticOperator())
@@ -1133,7 +1157,7 @@ AbstractMetaFunctionList AbstractMetaClass::operatorOverloads(uint query) const
 bool AbstractMetaClass::hasOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isOperatorOverload())
+        if (f->ownerClass() == f->implementingClass() && f->isOperatorOverload() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1142,7 +1166,7 @@ bool AbstractMetaClass::hasOperatorOverload() const
 bool AbstractMetaClass::hasArithmeticOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isArithmeticOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isArithmeticOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1151,7 +1175,7 @@ bool AbstractMetaClass::hasArithmeticOperatorOverload() const
 bool AbstractMetaClass::hasBitwiseOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isBitwiseOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isBitwiseOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1160,7 +1184,7 @@ bool AbstractMetaClass::hasBitwiseOperatorOverload() const
 bool AbstractMetaClass::hasComparisonOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isComparisonOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isComparisonOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1169,7 +1193,7 @@ bool AbstractMetaClass::hasComparisonOperatorOverload() const
 bool AbstractMetaClass::hasLogicalOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isLogicalOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isLogicalOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1178,7 +1202,7 @@ bool AbstractMetaClass::hasLogicalOperatorOverload() const
 bool AbstractMetaClass::hasSubscriptOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isSubscriptOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isSubscriptOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1187,7 +1211,7 @@ bool AbstractMetaClass::hasSubscriptOperatorOverload() const
 bool AbstractMetaClass::hasAssignmentOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isAssignmentOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isAssignmentOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1196,7 +1220,7 @@ bool AbstractMetaClass::hasAssignmentOperatorOverload() const
 bool AbstractMetaClass::hasConversionOperatorOverload() const
 {
     foreach (const AbstractMetaFunction *f, m_functions) {
-        if (f->isConversionOperator())
+        if (f->ownerClass() == f->implementingClass() && f->isConversionOperator() && !f->isPrivate())
             return true;
     }
     return false;
@@ -1259,10 +1283,10 @@ void AbstractMetaClass::setFunctions(const AbstractMetaFunctionList &functions)
     foreach (AbstractMetaFunction *f, m_functions) {
         f->setOwnerClass(this);
 
-        m_hasVirtualSlots |= f->isVirtualSlot();
-        m_hasVirtuals |= !f->isFinal() || f->isVirtualSlot();
-        m_isPolymorphic |= m_hasVirtuals;
-        m_hasNonpublic |= !f->isPublic();
+        m_hasVirtualSlots = m_hasVirtualSlots || f->isVirtualSlot();
+        m_hasVirtuals = m_hasVirtuals || f->isVirtualSlot() || hasVirtualDestructor();
+        m_isPolymorphic = m_isPolymorphic || m_hasVirtuals;
+        m_hasNonpublic = m_hasNonpublic || !f->isPublic();
 
         // If we have non-virtual overloads of a virtual function, we have to implement
         // all the overloads in the shell class to override the hiding rule
@@ -1315,9 +1339,11 @@ void AbstractMetaClass::addFunction(AbstractMetaFunction *function)
 
     if (!function->isDestructor())
         m_functions << function;
+    else
+        Q_ASSERT(false); //memory leak
 
     m_hasVirtualSlots |= function->isVirtualSlot();
-    m_hasVirtuals |= !function->isFinal() || function->isVirtualSlot();
+    m_hasVirtuals |= !function->isFinal() || function->isVirtualSlot() || hasVirtualDestructor();
     m_isPolymorphic |= m_hasVirtuals;
     m_hasNonpublic |= !function->isPublic();
 }
@@ -1362,11 +1388,6 @@ const AbstractMetaFunction* AbstractMetaClass::findFunction(const QString& funct
     return 0;
 }
 
-/* Returns true if this class has one or more functions that are
-   protected. If a class has protected members we need to generate a
-   shell class with public accessors to the protected functions, so
-   they can be called from the native functions.
-*/
 bool AbstractMetaClass::hasProtectedFunctions() const
 {
     foreach (AbstractMetaFunction *func, m_functions) {
@@ -1376,6 +1397,20 @@ bool AbstractMetaClass::hasProtectedFunctions() const
     return false;
 }
 
+bool AbstractMetaClass::hasProtectedFields() const
+{
+    foreach (const AbstractMetaField *field, fields()) {
+        if (field->isProtected())
+            return true;
+    }
+    return false;
+}
+
+bool AbstractMetaClass::hasProtectedMembers() const
+{
+    return hasProtectedFields() || hasProtectedFunctions();
+}
+
 bool AbstractMetaClass::generateShellClass() const
 {
     return m_forceShellClass ||
@@ -1430,7 +1465,7 @@ AbstractMetaField::~AbstractMetaField()
     delete m_setter;
     delete m_getter;
 }
-ushort        painters;                        // refcount
+
 AbstractMetaField *AbstractMetaField::copy() const
 {
     AbstractMetaField *returned = new AbstractMetaField;
@@ -1455,8 +1490,6 @@ static AbstractMetaFunction *createXetter(const AbstractMetaField *g, const QStr
 {
     AbstractMetaFunction *f = new AbstractMetaFunction;
 
-
-
     f->setName(name);
     f->setOriginalName(name);
     f->setOwnerClass(g->enclosingClass());
@@ -1542,6 +1575,24 @@ bool AbstractMetaClass::hasConstructors() const
     return queryFunctions(Constructors).size();
 }
 
+bool AbstractMetaClass::hasCopyConstructor() const
+{
+    foreach (const AbstractMetaFunction* ctor, queryFunctions(Constructors)) {
+        if (ctor->isCopyConstructor())
+            return true;
+    }
+    return false;
+}
+
+bool AbstractMetaClass::hasPrivateCopyConstructor() const
+{
+    foreach (const AbstractMetaFunction* ctor, queryFunctions(Constructors)) {
+        if (ctor->isCopyConstructor() && ctor->isPrivate())
+            return true;
+    }
+    return false;
+}
+
 void AbstractMetaClass::addDefaultConstructor()
 {
     AbstractMetaFunction *f = new AbstractMetaFunction;
@@ -1553,6 +1604,40 @@ void AbstractMetaClass::addDefaultConstructor()
 
     uint attr = AbstractMetaAttributes::Native;
     attr |= AbstractMetaAttributes::Public;
+    attr |= AbstractMetaAttributes::Final;
+    f->setAttributes(attr);
+    f->setImplementingClass(this);
+    f->setOriginalAttributes(f->attributes());
+
+    addFunction(f);
+    this->setHasNonPrivateConstructor(true);
+}
+
+void AbstractMetaClass::addDefaultCopyConstructor(bool isPrivate)
+{
+    AbstractMetaFunction* f = new AbstractMetaFunction;
+    f->setName(name());
+    f->setOwnerClass(this);
+    f->setFunctionType(AbstractMetaFunction::ConstructorFunction);
+    f->setDeclaringClass(this);
+
+    AbstractMetaType* argType = new AbstractMetaType;
+    argType->setTypeEntry(typeEntry());
+    argType->setReference(true);
+    argType->setConstant(true);
+    argType->setTypeUsagePattern(AbstractMetaType::ValuePattern);
+
+    AbstractMetaArgument* arg = new AbstractMetaArgument;
+    arg->setType(argType);
+    arg->setName(name());
+    f->addArgument(arg);
+
+    uint attr = AbstractMetaAttributes::Native;
+    attr |= AbstractMetaAttributes::Final;
+    if (isPrivate)
+        attr |= AbstractMetaAttributes::Private;
+    else
+        attr |= AbstractMetaAttributes::Public;
     f->setAttributes(attr);
     f->setImplementingClass(this);
     f->setOriginalAttributes(f->attributes());
@@ -1672,8 +1757,6 @@ AbstractMetaFunctionList AbstractMetaClass::queryFunctions(uint query) const
         functions << f;
     }
 
-//    qDebug() << "queried" << m_typeEntry->qualifiedCppName() << "got" << functions.size() << "out of" << m_functions.size();
-
     return functions;
 }
 
@@ -1820,9 +1903,9 @@ static void addExtraIncludeForType(AbstractMetaClass *metaClass, const AbstractM
     }
 
     if (type->hasInstantiations()) {
-        QList<AbstractMetaType *> instantiations = type->instantiations();
-        foreach (AbstractMetaType *instantiation, instantiations)
-        addExtraIncludeForType(metaClass, instantiation);
+        AbstractMetaTypeList instantiations = type->instantiations();
+        foreach (const AbstractMetaType *instantiation, instantiations)
+            addExtraIncludeForType(metaClass, instantiation);
     }
 }
 
@@ -1847,14 +1930,10 @@ void AbstractMetaClass::fixFunctions()
     AbstractMetaClass *superClass = baseClass();
     AbstractMetaFunctionList funcs = functions();
 
-//     printf("fix functions for %s\n", qPrintable(name()));
-
     if (superClass)
         superClass->fixFunctions();
     int iface_idx = 0;
     while (superClass || iface_idx < interfaces().size()) {
-//         printf(" - base: %s\n", qPrintable(super_class->name()));
-
         // Since we always traverse the complete hierarchy we are only
         // interrested in what each super class implements, not what
         // we may have propagated from their base classes again.
@@ -1892,35 +1971,16 @@ void AbstractMetaClass::fixFunctions()
                 uint cmp = f->compareTo(sf);
 
                 if (cmp & AbstractMetaFunction::EqualModifiedName) {
-//                     printf("   - %s::%s similar to %s::%s %x vs %x\n",
-//                            qPrintable(sf->implementingClass()->typeEntry()->qualifiedCppName()),
-//                            qPrintable(sf->name()),
-//                            qPrintable(f->implementingClass()->typeEntry()->qualifiedCppName()),
-//                            qPrintable(f->name()),
-//                            sf->attributes(),
-//                            f->attributes());
-
                     add = false;
                     if (cmp & AbstractMetaFunction::EqualArguments) {
-
-//                         if (!(cmp & AbstractMetaFunction::EqualReturnType)) {
-//                             ReportHandler::warning(QString("%1::%2 and %3::%4 differ in retur type")
-//                                                    .arg(sf->implementingClass()->name())
-//                                                    .arg(sf->name())
-//                                                    .arg(f->implementingClass()->name())
-//                                                    .arg(f->name()));
-//                         }
-
                         // Same function, propegate virtual...
                         if (!(cmp & AbstractMetaFunction::EqualAttributes)) {
                             if (!f->isEmptyFunction()) {
                                 if (!sf->isFinalInCpp() && f->isFinalInCpp()) {
                                     *f -= AbstractMetaAttributes::FinalInCpp;
-                                    //                                 printf("   --- inherit virtual\n");
                                 }
                                 if (!sf->isFinalInTargetLang() && f->isFinalInTargetLang()) {
                                     *f -= AbstractMetaAttributes::FinalInTargetLang;
-                                    //                                 printf("   --- inherit virtual\n");
                                 }
 #if 0
                                 if (!f->isFinalInTargetLang() && f->isPrivate()) {
@@ -1950,9 +2010,8 @@ void AbstractMetaClass::fixFunctions()
                             // Private overrides of abstract functions have to go into the class or
                             // the subclasses will not compile as non-abstract classes.
                             // But they don't need to be implemented, since they can never be called.
-                            if (f->isPrivate() && sf->isAbstract()) {
+                            if (f->isPrivate()) {
                                 f->setFunctionType(AbstractMetaFunction::EmptyFunction);
-//                                 f->setVisibility(sf->visibility());
                                 *f += AbstractMetaAttributes::FinalInTargetLang;
                                 *f += AbstractMetaAttributes::FinalInCpp;
                             }
@@ -2033,8 +2092,6 @@ void AbstractMetaClass::fixFunctions()
         FunctionModificationList mods = func->modifications(this);
         foreach (const FunctionModification &mod, mods) {
             if (mod.isRenameModifier()) {
-//                 qDebug() << name() << func->originalName() << func << " from "
-//                          << func->implementingClass()->name() << "renamed to" << mod.renamedTo();
                 func->setName(mod.renamedTo());
             }
         }
@@ -2073,16 +2130,6 @@ void AbstractMetaClass::fixFunctions()
                     && !f1->isFinalInCpp()
                     && f2->isFinalInCpp()) {
                     *f2 += AbstractMetaAttributes::FinalOverload;
-//                     qDebug() << f2 << f2->implementingClass()->name() << "::" << f2->name() << f2->arguments().size() << " vs " << f1 << f1->implementingClass()->name() << "::" << f1->name() << f1->arguments().size();
-//                     qDebug() << "    " << f2;
-//                     AbstractMetaArgumentList f2Args = f2->arguments();
-//                     foreach (AbstractMetaArgument *a, f2Args)
-//                         qDebug() << "        " << a->type()->name() << a->name();
-//                     qDebug() << "    " << f1;
-//                     AbstractMetaArgumentList f1Args = f1->arguments();
-//                     foreach (AbstractMetaArgument *a, f1Args)
-//                         qDebug() << "        " << a->type()->name() << a->name();
-
                 }
             }
         }
@@ -2099,20 +2146,20 @@ QString AbstractMetaType::minimalSignature() const
         minimalSignature += "const ";
     minimalSignature += typeEntry()->qualifiedCppName();
     if (hasInstantiations()) {
-        QList<AbstractMetaType *> instantiations = this->instantiations();
+        AbstractMetaTypeList instantiations = this->instantiations();
         minimalSignature += "< ";
         for (int i = 0; i < instantiations.size(); ++i) {
             if (i > 0)
                 minimalSignature += ",";
-            minimalSignature += instantiations.at(i)->minimalSignature();
+            minimalSignature += instantiations[i]->minimalSignature();
         }
         minimalSignature += " >";
     }
 
-    if (isReference())
-        minimalSignature += "&";
     for (int j = 0; j < indirections(); ++j)
         minimalSignature += "*";
+    if (isReference())
+        minimalSignature += "&";
 
     return minimalSignature;
 }
@@ -2210,3 +2257,12 @@ AbstractMetaClass *AbstractMetaClassList::findClass(const QString &name) const
 
     return 0;
 }
+
+AbstractMetaClass *AbstractMetaClassList::findClass(const TypeEntry* typeEntry) const
+{
+    foreach (AbstractMetaClass* c, *this) {
+        if (c->typeEntry() == typeEntry)
+            return c;
+    }
+    return 0;
+}
