@@ -29,6 +29,7 @@
 #include <QtCore/QSet>
 #include <QtCore/QStringList>
 #include <QtCore/QTextStream>
+#include <QSharedPointer>
 
 
 class AbstractMeta;
@@ -84,6 +85,7 @@ class APIEXTRACTOR_API AbstractMetaClassList : public  QList<AbstractMetaClass *
 {
 public:
     AbstractMetaClass *findClass(const QString &name) const;
+    AbstractMetaClass *findClass(const TypeEntry* typeEntry) const;
     AbstractMetaEnumValue *findEnumValue(const QString &string) const;
     AbstractMetaEnum *findEnum(const EnumTypeEntry *entry) const;
 
@@ -92,7 +94,7 @@ public:
 class APIEXTRACTOR_API AbstractMetaAttributes
 {
 public:
-    AbstractMetaAttributes() : m_attributes(0) {};
+    AbstractMetaAttributes() : m_attributes(0), m_originalAttributes(0) {};
 
     enum Attribute {
         None                        = 0x00000000,
@@ -273,18 +275,6 @@ public:
         return m_originalAttributes & Friendly;
     }
 
-    // valid only for global components
-    // (e.g.: functions, enums, flags)
-    QString includeFile() const
-    {
-        return m_includeFile;
-    }
-
-    void setIncludeFile(const QString& file)
-    {
-        m_includeFile = file;
-    }
-
     void setDocumentation(const Documentation& doc)
     {
         m_doc = doc;
@@ -298,14 +288,14 @@ public:
 private:
     uint m_attributes;
     uint m_originalAttributes;
-    QString m_includeFile;
     Documentation m_doc;
 };
 
-
+typedef QList<AbstractMetaType*> AbstractMetaTypeList;
 class APIEXTRACTOR_API AbstractMetaType
 {
 public:
+
     enum TypeUsagePattern {
         InvalidPattern,
         PrimitivePattern,
@@ -326,18 +316,8 @@ public:
         ThreadPattern
     };
 
-    AbstractMetaType() :
-            m_typeEntry(0),
-            m_arrayElementCount(0),
-            m_arrayElementType(0),
-            m_originalTemplateType(0),
-            m_pattern(InvalidPattern),
-            m_constant(false),
-            m_reference(false),
-            m_cppInstantiation(true),
-            m_indirections(0),
-            m_reserved(0)
-    {}
+    AbstractMetaType();
+    ~AbstractMetaType();
 
     QString package() const
     {
@@ -366,18 +346,28 @@ public:
     {
         return !m_instantiations.isEmpty();
     }
-    void addInstantiation(AbstractMetaType *inst)
+
+    void addInstantiation(AbstractMetaType* inst, bool owner = false)
     {
+        if (owner)
+            m_children << inst;
         m_instantiations << inst;
     }
-    void setInstantiations(const QList<AbstractMetaType *> &insts)
+
+    void setInstantiations(const AbstractMetaTypeList  &insts, bool owner = false)
     {
         m_instantiations = insts;
+        if (owner) {
+            m_children.clear();
+            m_children = insts;
+        }
     }
-    QList<AbstractMetaType *> instantiations() const
+
+    AbstractMetaTypeList instantiations() const
     {
         return m_instantiations;
     }
+
     void setInstantiationInCpp(bool incpp)
     {
         m_cppInstantiation = incpp;
@@ -557,11 +547,11 @@ public:
         return m_arrayElementCount;
     }
 
-    AbstractMetaType *arrayElementType() const
+    const AbstractMetaType *arrayElementType() const
     {
         return m_arrayElementType;
     }
-    void setArrayElementType(AbstractMetaType *t)
+    void setArrayElementType(const AbstractMetaType *t)
     {
         m_arrayElementType = t;
     }
@@ -599,12 +589,12 @@ public:
 
 private:
     const TypeEntry *m_typeEntry;
-    QList <AbstractMetaType *> m_instantiations;
+    AbstractMetaTypeList m_instantiations;
     QString m_package;
     QString m_originalTypeDescription;
 
     int m_arrayElementCount;
-    AbstractMetaType *m_arrayElementType;
+    const AbstractMetaType *m_arrayElementType;
     const AbstractMetaType *m_originalTemplateType;
 
     TypeUsagePattern m_pattern;
@@ -613,12 +603,21 @@ private:
     uint m_cppInstantiation : 1;
     int m_indirections : 4;
     uint m_reserved : 25; // unused
+    AbstractMetaTypeList m_children;
+
+    Q_DISABLE_COPY(AbstractMetaType);
 };
 
 class APIEXTRACTOR_API AbstractMetaVariable
 {
 public:
-    AbstractMetaVariable() : m_type(0) {}
+    AbstractMetaVariable() : m_type(0), m_hasName(false) {}
+    AbstractMetaVariable(const AbstractMetaVariable &other);
+
+    virtual ~AbstractMetaVariable()
+    {
+        delete m_type;
+    }
 
     AbstractMetaType *type() const
     {
@@ -626,6 +625,13 @@ public:
     }
     void setType(AbstractMetaType *type)
     {
+        Q_ASSERT(m_type == 0);
+        m_type = type;
+    }
+    void replaceType(AbstractMetaType *type)
+    {
+        if (m_type)
+            delete m_type;
         m_type = type;
     }
 
@@ -633,11 +639,23 @@ public:
     {
         return m_name;
     }
-    void setName(const QString &name)
+    void setName(const QString &name, bool realName = true)
     {
         m_name = name;
+        m_hasName = realName;
+    }
+    bool hasName() const
+    {
+        return m_hasName;
+    }
+    QString originalName() const
+    {
+        return m_originalName;
+    }
+    void setOriginalName(const QString& name)
+    {
+        m_originalName = name;
     }
-
     void setDocumentation(const Documentation& doc)
     {
         m_doc = doc;
@@ -648,8 +666,10 @@ public:
     }
 
 private:
+    QString m_originalName;
     QString m_name;
     AbstractMetaType *m_type;
+    bool m_hasName;
 
     Documentation m_doc;
 };
@@ -694,18 +714,13 @@ public:
         m_argumentIndex = argIndex;
     }
 
-    QString argumentName() const;
-    QString indexedName() const;
-
     AbstractMetaArgument *copy() const;
-
 private:
-    // Just to force people to call argumentName() And indexedName();
-    QString name() const;
-
     QString m_expression;
     QString m_originalExpression;
     int m_argumentIndex;
+
+    friend class AbstractMetaClass;
 };
 
 
@@ -770,7 +785,8 @@ public:
     };
 
     AbstractMetaFunction()
-            : m_functionType(NormalFunction),
+            : m_typeEntry(0),
+            m_functionType(NormalFunction),
             m_type(0),
             m_class(0),
             m_implementingClass(0),
@@ -893,6 +909,14 @@ public:
     }
     void setType(AbstractMetaType *type)
     {
+        Q_ASSERT(m_type == 0);
+        m_type = type;
+    }
+
+    void replaceType(AbstractMetaType *type)
+    {
+        if (m_type)
+            delete m_type;
         m_type = type;
     }
 
@@ -1079,6 +1103,11 @@ public:
     bool hasSignatureModifications() const;
     FunctionModificationList modifications(const AbstractMetaClass* implementor = 0) const;
 
+    /**
+     * Return the argument name if there is a modification the renamed value will be returned
+     */
+    QString argumentName(int index, bool create = true, const AbstractMetaClass *cl = 0) const;
+
     // If this function stems from an interface, this returns the
     // interface that declares it.
     const AbstractMetaClass *interfaceClass() const
@@ -1101,12 +1130,22 @@ public:
         return m_propertySpec;
     }
 
+    FunctionTypeEntry* typeEntry() const
+    {
+        return m_typeEntry;
+    }
+
+    void setTypeEntry(FunctionTypeEntry* typeEntry)
+    {
+        m_typeEntry = typeEntry;
+    }
 private:
     QString m_name;
     QString m_originalName;
     mutable QString m_cachedMinimalSignature;
     mutable QString m_cachedModifiedName;
 
+    FunctionTypeEntry* m_typeEntry;
     FunctionType m_functionType;
     AbstractMetaType *m_type;
     const AbstractMetaClass *m_class;
@@ -1198,6 +1237,10 @@ class AbstractMetaEnum : public AbstractMetaAttributes
 {
 public:
     AbstractMetaEnum() : m_typeEntry(0), m_class(0), m_hasQenumsDeclaration(false) {}
+    ~AbstractMetaEnum()
+    {
+        qDeleteAll(m_enumValues);
+    }
 
     AbstractMetaEnumValueList values() const
     {
@@ -1342,7 +1385,8 @@ public:
               m_typeEntry(0),
               //m_qDebugStreamFunction(0)
               m_stream(false)
-    {}
+    {
+    }
 
     virtual ~AbstractMetaClass();
 
@@ -1362,8 +1406,11 @@ public:
     bool hasSignal(const AbstractMetaFunction *f) const;
 
     bool hasConstructors() const;
+    bool hasCopyConstructor() const;
+    bool hasPrivateCopyConstructor() const;
 
     void addDefaultConstructor();
+    void addDefaultCopyConstructor(bool isPrivate = false);
 
     bool hasNonPrivateConstructor() const
     {
@@ -1421,7 +1468,7 @@ public:
 
     /**
      *   Retrieves all class' operator overloads that meet
-     *   query crieteria defined with the OperatorQueryOption
+     *   query criteria defined with the OperatorQueryOption
      *   enum.
      *   /param query composition of OperatorQueryOption enum values
      *   /return list of operator overload methods that meet the
@@ -1588,8 +1635,26 @@ public:
     {
         return m_isPolymorphic;
     }
+
+    /**
+     * Tells if this class has one or more functions that are protected.
+     * \return true if the class has protected functions.
+     */
     bool hasProtectedFunctions() const;
 
+    /**
+     * Tells if this class has one or more fields (member variables) that are protected.
+     * \return true if the class has protected fields.
+     */
+    bool hasProtectedFields() const;
+
+    /**
+     * Tells if this class has one or more members (functions or fields) that are protected.
+     * \return true if the class has protected members.
+     */
+    bool hasProtectedMembers() const;
+
+
     QList<TypeEntry *> templateArguments() const
     {
         return m_templateArgs;
@@ -1756,6 +1821,23 @@ public:
         return m_greaterThanEqFunctions;
     }
 
+    /// Returns a list of conversion operators for this class. The conversion operators are defined in other classes of the same module.
+    AbstractMetaFunctionList externalConversionOperators() const
+    {
+        return m_externalConversionOperators;
+    }
+    /// Adds a converter operator for this class.
+    void addExternalConversionOperator(AbstractMetaFunction* conversionOp)
+    {
+        if (!m_externalConversionOperators.contains(conversionOp))
+            m_externalConversionOperators.append(conversionOp);
+    }
+    /// Returns true if this class has any converter operators defined elsewhere.
+    bool hasExternalConversionOperators() const
+    {
+        return !m_externalConversionOperators.isEmpty();
+    }
+
     void sortFunctions();
 
     const AbstractMetaClass *templateBaseClass() const
@@ -1814,6 +1896,7 @@ private:
     AbstractMetaFieldList m_fields;
     AbstractMetaEnumList m_enums;
     AbstractMetaClassList m_interfaces;
+    AbstractMetaClassList m_orphanInterfaces;
     AbstractMetaClass *m_extractedInterface;
     AbstractMetaClass *m_primaryInterfaceImplementor;
     QList<QPropertySpec *> m_propertySpecs;
@@ -1826,12 +1909,15 @@ private:
     AbstractMetaFunctionList m_lessThanEqFunctions;
     AbstractMetaFunctionList m_greaterThanEqFunctions;
 
+    AbstractMetaFunctionList m_externalConversionOperators;
+
     QStringList m_baseClassNames;
     QList<TypeEntry *> m_templateArgs;
     ComplexTypeEntry *m_typeEntry;
 //     FunctionModelItem m_qDebugStreamFunction;
 
     bool m_stream;
+    static int m_count;
 };
 
 class QPropertySpec
