@@ -38,6 +38,10 @@ private slots:
     void testAddFunctionWithDefaultArgs();
     void testAddFunctionAtModuleLevel();
     void testAddFunctionWithVarargs();
+    void testAddStaticFunction();
+    void testAddGlobalFunction();
+    void testAddFunctionWithApiVersion();
+    void testModifyAddedFunction();
 };
 
 #endif
