@@ -30,6 +30,7 @@
 #include <QtCore/QMap>
 #include <QtCore/QDebug>
 #include "apiextractormacros.h"
+#include "include.h"
 
 class Indentor;
 
@@ -39,34 +40,6 @@ class QTextStream;
 class EnumTypeEntry;
 class FlagsTypeEntry;
 
-struct APIEXTRACTOR_API Include
-{
-    enum IncludeType {
-        IncludePath,
-        LocalPath,
-        TargetLangImport
-    };
-
-    Include() : type(IncludePath) {}
-    Include(IncludeType t, const QString &nam) : type(t), name(nam) {};
-
-    bool isValid()
-    {
-        return !name.isEmpty();
-    }
-
-    IncludeType type;
-    QString name;
-
-    QString toString() const;
-
-    bool operator<(const Include &other) const
-    {
-        return name < other.name;
-    }
-};
-typedef QList<Include> IncludeList;
-
 typedef QMap<int, QString> ArgumentMap;
 
 class TemplateInstance;
@@ -122,6 +95,7 @@ struct ReferenceCount
     };
 
     Action action;
+    QString varName;
 };
 
 struct ArgumentOwner
@@ -140,7 +114,7 @@ struct ArgumentOwner
 class APIEXTRACTOR_API CodeSnipFragment
 {
 private:
-    const QString m_code;
+    QString m_code;
     TemplateInstance *m_instance;
 
 public:
@@ -161,15 +135,15 @@ public:
 
     void addCode(const QString &code)
     {
-        codeList.append(new CodeSnipFragment(code));
+        codeList.append(CodeSnipFragment(code));
     }
 
     void addTemplateInstance(TemplateInstance *ti)
     {
-        codeList.append(new CodeSnipFragment(ti));
+        codeList.append(CodeSnipFragment(ti));
     }
 
-    QList<CodeSnipFragment*> codeList;
+    QList<CodeSnipFragment> codeList;
 };
 
 class CustomFunction : public CodeSnipAbstract
@@ -184,8 +158,8 @@ public:
 class TemplateEntry : public CodeSnipAbstract
 {
 public:
-    TemplateEntry(const QString &name)
-            : m_name(name)
+    TemplateEntry(const QString &name, double vr)
+            : m_name(name), m_version(vr)
     {
     };
 
@@ -194,8 +168,14 @@ public:
         return m_name;
     };
 
+    double version() const
+    {
+        return m_version;
+    }
+
 private:
     QString m_name;
+    double m_version;
 };
 
 typedef QHash<QString, TemplateEntry *> TemplateEntryHash;
@@ -203,8 +183,8 @@ typedef QHash<QString, TemplateEntry *> TemplateEntryHash;
 class TemplateInstance
 {
 public:
-    TemplateInstance(const QString &name)
-            : m_name(name) {}
+    TemplateInstance(const QString &name, double vr)
+            : m_name(name), m_version(vr) {}
 
     void addReplaceRule(const QString &name, const QString &value)
     {
@@ -218,8 +198,14 @@ public:
         return m_name;
     }
 
+    double version() const
+    {
+        return m_version;
+    }
+
 private:
     const QString m_name;
+    double m_version;
     QHash<QString, QString> replaceRules;
 };
 
@@ -239,20 +225,21 @@ public:
         Any
     };
 
-    CodeSnip() : language(TypeSystem::TargetLangCode) { }
-    CodeSnip(TypeSystem::Language lang) : language(lang) { }
+    CodeSnip(double vr) : language(TypeSystem::TargetLangCode), version(vr) { }
+    CodeSnip(double vr, TypeSystem::Language lang) : language(lang), version(vr) { }
 
     TypeSystem::Language language;
     Position position;
     ArgumentMap argumentMap;
+    double version;
 };
 typedef QList<CodeSnip> CodeSnipList;
 
 struct ArgumentModification
 {
-    ArgumentModification(int idx)
+    ArgumentModification(int idx, double vr)
             : removedDefaultExpression(false), removed(false),
-              noNullPointers(false), index(idx) {}
+              noNullPointers(false), index(idx), version(vr) {}
 
     // Should the default expression be removed?
     uint removedDefaultExpression : 1;
@@ -288,6 +275,12 @@ struct ArgumentModification
 
     //QObject parent(owner) of this argument
     ArgumentOwner owner;
+
+    //Api version
+    double version;
+
+    //New name
+    QString renamed_to;
 };
 
 struct APIEXTRACTOR_API Modification
@@ -377,7 +370,7 @@ struct APIEXTRACTOR_API Modification
 
 struct APIEXTRACTOR_API FunctionModification: public Modification
 {
-    FunctionModification() : removal(TypeSystem::NoLanguage), m_thread(false), m_allowThread(false) {}
+    FunctionModification(double vr) : removal(TypeSystem::NoLanguage), m_thread(false), m_allowThread(false), m_version(vr) {}
 
     bool isCodeInjection() const
     {
@@ -403,6 +396,14 @@ struct APIEXTRACTOR_API FunctionModification: public Modification
     {
         m_allowThread = allow;
     }
+    double version() const
+    {
+        return m_version;
+    }
+
+    bool operator!=(const FunctionModification& other) const;
+    bool operator==(const FunctionModification& other) const;
+
 
     QString toString() const;
 
@@ -414,8 +415,12 @@ struct APIEXTRACTOR_API FunctionModification: public Modification
     QList<ArgumentModification> argument_mods;
 
 private:
+    FunctionModification() {}
+
     bool m_thread;
     bool m_allowThread;
+    double m_version;
+
 
 };
 typedef QList<FunctionModification> FunctionModificationList;
@@ -466,7 +471,7 @@ struct APIEXTRACTOR_API AddedFunction
     };
 
     /// Creates a new AddedFunction with a signature and a return type.
-    AddedFunction(QString signature, QString returnType);
+    AddedFunction(QString signature, QString returnType, double vr);
 
     /// Returns the function name.
     QString name() const
@@ -504,12 +509,30 @@ struct APIEXTRACTOR_API AddedFunction
         return m_isConst;
     }
 
+    /// Set this method static.
+    void setStatic(bool value)
+    {
+        m_isStatic = value;
+    }
+
+    /// Returns true if this is a static method.
+    bool isStatic() const
+    {
+        return m_isStatic;
+    }
+
+    double version() const
+    {
+        return m_version;
+    }
 private:
     QString m_name;
     Access m_access;
     QList<TypeInfo> m_arguments;
     TypeInfo m_returnType;
     bool m_isConst;
+    bool m_isStatic;
+    double m_version;
 };
 typedef QList<AddedFunction> AddedFunctionList;
 
@@ -537,11 +560,11 @@ public:
         XPathReplace
     };
 
-    DocModification(const QString& xpath, const QString& signature)
+    DocModification(const QString& xpath, const QString& signature, double vr)
             : format(TypeSystem::NativeCode), m_mode(XPathReplace),
-              m_xpath(xpath), m_signature(signature) {}
-    DocModification(Mode mode, const QString& signature)
-            : m_mode(mode), m_signature(signature) {}
+              m_xpath(xpath), m_signature(signature), m_version(vr) {}
+    DocModification(Mode mode, const QString& signature, double vr)
+            : m_mode(mode), m_signature(signature), m_version(vr) {}
 
     void setCode(const QString& code)
     {
@@ -563,6 +586,10 @@ public:
     {
         return m_mode;
     }
+    double version() const
+    {
+        return m_version;
+    }
 
     TypeSystem::Language format;
 
@@ -571,6 +598,7 @@ private:
     QString m_code;
     QString m_xpath;
     QString m_signature;
+    double m_version;
 };
 
 typedef QList<DocModification> DocModificationList;
@@ -584,6 +612,7 @@ public:
         VarargsType,
         FlagsType,
         EnumType,
+        EnumValue,
         TemplateArgumentType,
         ThreadType,
         BasicValueType,
@@ -598,7 +627,8 @@ public:
         ArrayType,
         TypeSystemType,
         CustomType,
-        TargetLangType
+        TargetLangType,
+        FunctionType
     };
 
     enum CodeGeneration {
@@ -611,12 +641,13 @@ public:
         GenerateCode            = GenerateTargetLang | GenerateCpp
     };
 
-    TypeEntry(const QString &name, Type t)
+    TypeEntry(const QString &name, Type t, double vr)
             : m_name(name),
               m_type(t),
               m_codeGeneration(GenerateAll),
               m_preferredConversion(true),
-              m_stream(false)
+              m_stream(false),
+              m_version(vr)
     {
     };
 
@@ -702,6 +733,14 @@ public:
     {
         return m_type == TypeSystemType;
     }
+    bool isFunction() const
+    {
+        return m_type == FunctionType;
+    }
+    bool isEnumValue() const
+    {
+        return m_type == EnumValue;
+    }
 
     virtual bool preferredConversion() const
     {
@@ -830,10 +869,7 @@ public:
         return false;
     }
 
-    CodeSnipList codeSnips() const
-    {
-        return m_codeSnips;
-    }
+    CodeSnipList codeSnips() const;
     void setCodeSnips(const CodeSnipList &codeSnips)
     {
         m_codeSnips = codeSnips;
@@ -862,9 +898,9 @@ public:
     }
     void addExtraInclude(const Include &include)
     {
-        if (!m_includesUsed.value(include.name, false)) {
+        if (!m_includesUsed.value(include.name(), false)) {
             m_extraIncludes << include;
-            m_includesUsed[include.name] = true;
+            m_includesUsed[include.name()] = true;
         }
     }
 
@@ -895,6 +931,12 @@ public:
         return !m_conversionRule.isEmpty();
     }
 
+    double version() const
+    {
+        return m_version;
+    }
+
+
 private:
     QString m_name;
     Type m_type;
@@ -909,6 +951,7 @@ private:
     QHash<QString, bool> m_includesUsed;
     QString m_conversionRule;
     bool m_stream;
+    double m_version;
 };
 typedef QHash<QString, QList<TypeEntry *> > TypeEntryHash;
 typedef QHash<QString, TypeEntry *> SingleTypeEntryHash;
@@ -917,8 +960,8 @@ typedef QHash<QString, TypeEntry *> SingleTypeEntryHash;
 class APIEXTRACTOR_API TypeSystemTypeEntry : public TypeEntry
 {
 public:
-    TypeSystemTypeEntry(const QString &name)
-            : TypeEntry(name, TypeSystemType)
+    TypeSystemTypeEntry(const QString &name, double vr)
+            : TypeEntry(name, TypeSystemType, vr)
     {
     };
 };
@@ -926,20 +969,20 @@ public:
 class APIEXTRACTOR_API VoidTypeEntry : public TypeEntry
 {
 public:
-    VoidTypeEntry() : TypeEntry("void", VoidType) { }
+    VoidTypeEntry() : TypeEntry("void", VoidType, 0) { }
 };
 
 class APIEXTRACTOR_API VarargsTypeEntry : public TypeEntry
 {
 public:
-    VarargsTypeEntry() : TypeEntry("...", VarargsType) { }
+    VarargsTypeEntry() : TypeEntry("...", VarargsType, 0) { }
 };
 
 class APIEXTRACTOR_API TemplateArgumentEntry : public TypeEntry
 {
 public:
-    TemplateArgumentEntry(const QString &name)
-            : TypeEntry(name, TemplateArgumentType), m_ordinal(0)
+    TemplateArgumentEntry(const QString &name, double vr)
+            : TypeEntry(name, TemplateArgumentType, vr), m_ordinal(0)
     {
     }
 
@@ -959,8 +1002,8 @@ private:
 class APIEXTRACTOR_API ArrayTypeEntry : public TypeEntry
 {
 public:
-    ArrayTypeEntry(const TypeEntry *nested_type)
-            : TypeEntry("Array", ArrayType), m_nestedType(nested_type)
+    ArrayTypeEntry(const TypeEntry *nested_type, double vr)
+            : TypeEntry("Array", ArrayType, vr), m_nestedType(nested_type)
     {
         Q_ASSERT(m_nestedType);
     }
@@ -994,8 +1037,8 @@ private:
 class APIEXTRACTOR_API PrimitiveTypeEntry : public TypeEntry
 {
 public:
-    PrimitiveTypeEntry(const QString &name)
-            : TypeEntry(name, PrimitiveType),
+    PrimitiveTypeEntry(const QString &name, double vr)
+            : TypeEntry(name, PrimitiveType, vr),
               m_preferredConversion(true),
               m_preferredTargetLangType(true),
               m_aliasedTypeEntry(0)
@@ -1020,6 +1063,19 @@ public:
         m_targetLangApiName = targetLangApiName;
     }
 
+    QString defaultConstructor() const
+    {
+        return m_defaultConstructor;
+    }
+    void setDefaultConstructor(const QString& defaultConstructor)
+    {
+        m_defaultConstructor = defaultConstructor;
+    }
+    bool hasDefaultConstructor() const
+    {
+        return !m_defaultConstructor.isEmpty();
+    }
+
     /**
      *   The PrimitiveTypeEntry pointed by this type entry if it
      *   represents an alias (i.e. a typedef).
@@ -1045,13 +1101,6 @@ public:
      */
     PrimitiveTypeEntry* basicAliasedTypeEntry() const;
 
-    QString javaObjectFullName() const
-    {
-        return javaObjectPackage() + "." + javaObjectName();
-    }
-    QString javaObjectName() const;
-    QString javaObjectPackage() const;
-
     virtual bool preferredConversion() const
     {
         return m_preferredConversion;
@@ -1073,6 +1122,7 @@ public:
 private:
     QString m_targetLangName;
     QString m_targetLangApiName;
+    QString m_defaultConstructor;
     uint m_preferredConversion : 1;
     uint m_preferredTargetLangType : 1;
     PrimitiveTypeEntry* m_aliasedTypeEntry;
@@ -1094,9 +1144,9 @@ struct EnumValueRedirection
 class APIEXTRACTOR_API EnumTypeEntry : public TypeEntry
 {
 public:
-    EnumTypeEntry(const QString &nspace, const QString &enumName)
+    EnumTypeEntry(const QString &nspace, const QString &enumName, double vr)
             : TypeEntry(nspace.isEmpty() ? enumName : nspace + QLatin1String("::") + enumName,
-                        EnumType),
+                        EnumType, vr),
             m_flags(0),
             m_extensible(false)
     {
@@ -1232,10 +1282,25 @@ private:
     bool m_forceInteger;
 };
 
+class APIEXTRACTOR_API EnumValueTypeEntry : public TypeEntry
+{
+public:
+    EnumValueTypeEntry(const QString& name, const QString& value, const EnumTypeEntry* enclosingEnum, double vr)
+        : TypeEntry(name, TypeEntry::EnumValue, vr), m_value(value), m_enclosingEnum(enclosingEnum)
+    {
+    }
+
+    QString value() const { return m_value; }
+    const EnumTypeEntry* enclosingEnum() const { return m_enclosingEnum; }
+private:
+    QString m_value;
+    const EnumTypeEntry* m_enclosingEnum;
+};
+
 class APIEXTRACTOR_API FlagsTypeEntry : public TypeEntry
 {
 public:
-    FlagsTypeEntry(const QString &name) : TypeEntry(name, FlagsType), m_enum(0)
+    FlagsTypeEntry(const QString &name, double vr) : TypeEntry(name, FlagsType, vr), m_enum(0)
     {
     }
 
@@ -1310,15 +1375,16 @@ public:
         Unknown
     };
 
-    ComplexTypeEntry(const QString &name, Type t)
-            : TypeEntry(QString(name).replace(".*::", ""), t),
+    ComplexTypeEntry(const QString &name, Type t, double vr)
+            : TypeEntry(QString(name).replace(".*::", ""), t, vr),
             m_qualifiedCppName(name),
             m_qobject(false),
             m_polymorphicBase(false),
             m_genericClass(false),
             m_typeFlags(0),
             m_copyableFlag(Unknown),
-            m_hashFunction("")
+            m_hashFunction(""),
+            m_baseContainerType(0)
     {
     }
 
@@ -1329,7 +1395,7 @@ public:
 
     ComplexTypeEntry *copy() const
     {
-        ComplexTypeEntry *centry = new ComplexTypeEntry(name(), type());
+        ComplexTypeEntry *centry = new ComplexTypeEntry(name(), type(), version());
         centry->setInclude(include());
         centry->setExtraIncludes(extraIncludes());
         centry->setAddedFunctions(addedFunctions());
@@ -1339,6 +1405,7 @@ public:
         centry->setDefaultSuperclass(defaultSuperclass());
         centry->setCodeSnips(codeSnips());
         centry->setTargetLangPackage(targetLangPackage());
+        centry->setBaseContainerType(baseContainerType());
 
         return centry;
     }
@@ -1519,6 +1586,16 @@ public:
         m_hashFunction = hashFunction;
     }
 
+    void setBaseContainerType(const ComplexTypeEntry *baseContainer)
+    {
+        m_baseContainerType = baseContainer;
+    }
+
+    const ComplexTypeEntry* baseContainerType() const
+    {
+        return m_baseContainerType;
+    }
+
 
 private:
     AddedFunctionList m_addedFunctions;
@@ -1541,6 +1618,8 @@ private:
     TypeFlags m_typeFlags;
     CopyableFlag m_copyableFlag;
     QString m_hashFunction;
+
+    const ComplexTypeEntry* m_baseContainerType;
 };
 
 class APIEXTRACTOR_API ContainerTypeEntry : public ComplexTypeEntry
@@ -1562,8 +1641,8 @@ public:
         PairContainer,
     };
 
-    ContainerTypeEntry(const QString &name, Type type)
-        : ComplexTypeEntry(name, ContainerType), m_type(type)
+    ContainerTypeEntry(const QString &name, Type type, double vr)
+        : ComplexTypeEntry(name, ContainerType, vr), m_type(type)
     {
         setCodeGeneration(GenerateForSubclass);
     }
@@ -1572,6 +1651,8 @@ public:
     {
         return m_type;
     }
+
+    QString typeName() const;
     QString targetLangName() const;
     QString targetLangPackage() const;
     QString qualifiedCppName() const;
@@ -1605,14 +1686,14 @@ typedef QList<const ContainerTypeEntry*> ContainerTypeEntryList;
 class APIEXTRACTOR_API NamespaceTypeEntry : public ComplexTypeEntry
 {
 public:
-    NamespaceTypeEntry(const QString &name) : ComplexTypeEntry(name, NamespaceType) { }
+    NamespaceTypeEntry(const QString &name, double vr) : ComplexTypeEntry(name, NamespaceType, vr) { }
 };
 
 
 class ValueTypeEntry : public ComplexTypeEntry
 {
 public:
-    ValueTypeEntry(const QString &name) : ComplexTypeEntry(name, BasicValueType) { }
+    ValueTypeEntry(const QString &name, double vr) : ComplexTypeEntry(name, BasicValueType, vr) { }
 
     bool isValue() const
     {
@@ -1625,15 +1706,15 @@ public:
     }
 
 protected:
-    ValueTypeEntry(const QString &name, Type t) : ComplexTypeEntry(name, t) { }
+    ValueTypeEntry(const QString &name, Type t, double vr) : ComplexTypeEntry(name, t, vr) { }
 };
 
 
 class StringTypeEntry : public ValueTypeEntry
 {
 public:
-    StringTypeEntry(const QString &name)
-            : ValueTypeEntry(name, StringType)
+    StringTypeEntry(const QString &name, double vr)
+            : ValueTypeEntry(name, StringType, vr)
     {
         setCodeGeneration(GenerateNothing);
     }
@@ -1651,7 +1732,7 @@ public:
 class CharTypeEntry : public ValueTypeEntry
 {
 public:
-    CharTypeEntry(const QString &name) : ValueTypeEntry(name, CharType)
+    CharTypeEntry(const QString &name, double vr) : ValueTypeEntry(name, CharType, vr)
     {
         setCodeGeneration(GenerateNothing);
     }
@@ -1672,7 +1753,7 @@ public:
 class VariantTypeEntry: public ValueTypeEntry
 {
 public:
-    VariantTypeEntry(const QString &name) : ValueTypeEntry(name, VariantType) { }
+    VariantTypeEntry(const QString &name, double vr) : ValueTypeEntry(name, VariantType, vr) { }
 
     QString targetLangApiName() const;
     QString targetLangName() const;
@@ -1688,8 +1769,8 @@ public:
 class APIEXTRACTOR_API InterfaceTypeEntry : public ComplexTypeEntry
 {
 public:
-    InterfaceTypeEntry(const QString &name)
-            : ComplexTypeEntry(name, InterfaceType) {}
+    InterfaceTypeEntry(const QString &name, double vr)
+            : ComplexTypeEntry(name, InterfaceType, vr) {}
 
     static QString interfaceName(const QString &name)
     {
@@ -1719,11 +1800,37 @@ private:
 };
 
 
+class APIEXTRACTOR_API FunctionTypeEntry : public TypeEntry
+{
+public:
+    FunctionTypeEntry(const QString& name, const QString& signature, double vr)
+            : TypeEntry(name, FunctionType, vr)
+    {
+        addSignature(signature);
+    }
+    void addSignature(const QString& signature)
+    {
+        m_signatures << signature;
+    }
+
+    QStringList signatures() const
+    {
+        return m_signatures;
+    }
+
+    bool hasSignature(const QString& signature) const
+    {
+        return m_signatures.contains(signature);
+    }
+private:
+    QStringList m_signatures;
+};
+
 class APIEXTRACTOR_API ObjectTypeEntry : public ComplexTypeEntry
 {
 public:
-    ObjectTypeEntry(const QString &name)
-            : ComplexTypeEntry(name, ObjectType), m_interface(0) {}
+    ObjectTypeEntry(const QString &name, double vr)
+            : ComplexTypeEntry(name, ObjectType, vr), m_interface(0) {}
 
     InterfaceTypeEntry *designatedInterface() const
     {
@@ -1751,271 +1858,6 @@ struct TypeRejection
     QString enum_name;
 };
 
-class APIEXTRACTOR_API TypeDatabase
-{
-    TypeDatabase();
-    TypeDatabase(const TypeDatabase&);
-    TypeDatabase& operator=(const TypeDatabase&);
-public:
-
-    /**
-    * Return the type system instance.
-    * \param newInstance This parameter is usefull just for unit testing, because singletons causes
-    *                    too many side effects on unit testing.
-    */
-    static TypeDatabase *instance(bool newInstance = false);
-
-    static QString normalizedSignature(const char* signature);
-
-    QStringList requiredTargetImports()
-    {
-        return m_requiredTargetImports;
-    }
-
-    void addRequiredTargetImport(const QString &moduleName)
-    {
-        if (!m_requiredTargetImports.contains(moduleName))
-            m_requiredTargetImports << moduleName;
-    }
-
-    QStringList typesystemPaths()
-    {
-        return m_typesystemPaths;
-    }
-    void addTypesystemPath(const QString &typesystem_paths)
-    {
-#if defined(Q_OS_WIN32)
-        char *path_splitter = const_cast<char *>(";");
-#else
-        char *path_splitter = const_cast<char *>(":");
-#endif
-        m_typesystemPaths += typesystem_paths.split(path_splitter);
-    }
-
-    IncludeList extraIncludes(const QString &className);
-
-    inline PrimitiveTypeEntry *findPrimitiveType(const QString &name);
-    inline ComplexTypeEntry *findComplexType(const QString &name);
-    inline ObjectTypeEntry *findObjectType(const QString &name);
-    inline NamespaceTypeEntry *findNamespaceType(const QString &name);
-    ContainerTypeEntry *findContainerType(const QString &name);
-
-    TypeEntry *findType(const QString &name) const
-    {
-        QList<TypeEntry *> entries = findTypes(name);
-        foreach (TypeEntry *entry, entries) {
-            if (entry &&
-                (!entry->isPrimitive() || static_cast<PrimitiveTypeEntry *>(entry)->preferredTargetLangType())) {
-                return entry;
-            }
-        }
-        return 0;
-    }
-    QList<TypeEntry *> findTypes(const QString &name) const
-    {
-        return m_entries.value(name);
-    }
-    TypeEntryHash allEntries()
-    {
-        return m_entries;
-    }
-
-    SingleTypeEntryHash entries()
-    {
-        TypeEntryHash entries = allEntries();
-
-        SingleTypeEntryHash returned;
-        QList<QString> keys = entries.keys();
-
-        foreach (QString key, keys)
-            returned[key] = findType(key);
-
-        return returned;
-    }
-
-    PrimitiveTypeEntry *findTargetLangPrimitiveType(const QString &targetLangName);
-
-    QList<const PrimitiveTypeEntry*> primitiveTypes() {
-        TypeEntryHash entries = allEntries();
-        QList<const PrimitiveTypeEntry*> returned;
-        foreach(QString key, entries.keys()) {
-            foreach(const TypeEntry* typeEntry, entries[key]) {
-                if (typeEntry->isPrimitive())
-                    returned.append((PrimitiveTypeEntry*) typeEntry);
-            }
-        }
-        return returned;
-    }
-    QList<const ContainerTypeEntry*> containerTypes() {
-        TypeEntryHash entries = allEntries();
-        QList<const ContainerTypeEntry*> returned;
-        foreach(QString key, entries.keys()) {
-            foreach(const TypeEntry* typeEntry, entries[key]) {
-                if (typeEntry->isContainer())
-                    returned.append((ContainerTypeEntry*) typeEntry);
-            }
-        }
-        return returned;
-    }
-
-    void addRejection(const QString &class_name, const QString &function_name,
-                      const QString &field_name, const QString &enum_name);
-    bool isClassRejected(const QString &class_name);
-    bool isFunctionRejected(const QString &class_name, const QString &function_name);
-    bool isFieldRejected(const QString &class_name, const QString &field_name);
-    bool isEnumRejected(const QString &class_name, const QString &enum_name);
-
-    void addType(TypeEntry *e)
-    {
-        m_entries[e->qualifiedCppName()].append(e);
-    }
-
-    SingleTypeEntryHash flagsEntries() const
-    {
-        return m_flagsEntries;
-    }
-    FlagsTypeEntry *findFlagsType(const QString &name) const;
-    void addFlagsType(FlagsTypeEntry *fte)
-    {
-        m_flagsEntries[fte->originalName()] = fte;
-    }
-
-    TemplateEntry *findTemplate(const QString &name)
-    {
-        return m_templates[name];
-    }
-    void addTemplate(TemplateEntry *t)
-    {
-        m_templates[t->name()] = t;
-    }
-
-    AddedFunctionList addedFunctions() const
-    {
-        return m_addedFunctions;
-    }
-    void setAddedFunctions(const AddedFunctionList& addedFunctions)
-    {
-        m_addedFunctions = addedFunctions;
-    }
-    AddedFunctionList findAddedFunctions(const QString& name) const;
-
-    void setFunctionModifications(const FunctionModificationList& functionModifications)
-    {
-        m_functionMods = functionModifications;
-    }
-    void addFunctionModification(const FunctionModification& functionModification)
-    {
-        m_functionMods << functionModification;
-    }
-    FunctionModificationList functionModifications(const QString& signature) const;
-
-    void setSuppressWarnings(bool on)
-    {
-        m_suppressWarnings = on;
-    }
-    void addSuppressedWarning(const QString &s)
-    {
-        m_suppressedWarnings.append(s);
-    }
-
-    bool isSuppressedWarning(const QString &s)
-    {
-        if (!m_suppressWarnings)
-            return false;
-
-        foreach (const QString &_warning, m_suppressedWarnings) {
-            QString warning(QString(_warning).replace("\\*", "&place_holder_for_asterisk;"));
-
-            QStringList segs = warning.split("*", QString::SkipEmptyParts);
-            if (!segs.size())
-                continue;
-
-            int i = 0;
-            int pos = s.indexOf(QString(segs.at(i++)).replace("&place_holder_for_asterisk;", "*"));
-            //qDebug() << "s == " << s << ", warning == " << segs;
-            while (pos != -1) {
-                if (i == segs.size())
-                    return true;
-                pos = s.indexOf(QString(segs.at(i++)).replace("&place_holder_for_asterisk;", "*"), pos);
-            }
-        }
-
-        return false;
-    }
-
-    void setRebuildClasses(const QStringList &cls)
-    {
-        m_rebuildClasses = cls;
-    }
-
-    static QString globalNamespaceClassName(const TypeEntry *te);
-    QString filename() const
-    {
-        return "typesystem.txt";
-    }
-
-    QString modifiedTypesystemFilepath(const QString &ts_file);
-    bool parseFile(const QString &filename, bool generate = true);
-    bool parseFile(QIODevice* device, bool generate = true);
-
-private:
-    bool m_suppressWarnings;
-    TypeEntryHash m_entries;
-    SingleTypeEntryHash m_flagsEntries;
-    TemplateEntryHash m_templates;
-    QStringList m_suppressedWarnings;
-
-    AddedFunctionList m_addedFunctions;
-    FunctionModificationList m_functionMods;
-
-    QStringList m_requiredTargetImports;
-
-    QStringList m_typesystemPaths;
-    QHash<QString, bool> m_parsedTypesystemFiles;
-
-    QList<TypeRejection> m_rejections;
-    QStringList m_rebuildClasses;
-};
-
-inline PrimitiveTypeEntry *TypeDatabase::findPrimitiveType(const QString &name)
-{
-    QList<TypeEntry *> entries = findTypes(name);
-
-    foreach (TypeEntry *entry, entries) {
-        if (entry && entry->isPrimitive() && static_cast<PrimitiveTypeEntry *>(entry)->preferredTargetLangType())
-            return static_cast<PrimitiveTypeEntry *>(entry);
-    }
-
-    return 0;
-}
-
-inline ComplexTypeEntry *TypeDatabase::findComplexType(const QString &name)
-{
-    TypeEntry *entry = findType(name);
-    if (entry && entry->isComplex())
-        return static_cast<ComplexTypeEntry *>(entry);
-    else
-        return 0;
-}
-
-inline ObjectTypeEntry *TypeDatabase::findObjectType(const QString &name)
-{
-    TypeEntry *entry = findType(name);
-    if (entry && entry->isObject())
-        return static_cast<ObjectTypeEntry *>(entry);
-    else
-        return 0;
-}
-
-inline NamespaceTypeEntry *TypeDatabase::findNamespaceType(const QString &name)
-{
-    TypeEntry *entry = findType(name);
-    if (entry && entry->isNamespace())
-        return static_cast<NamespaceTypeEntry *>(entry);
-    else
-        return 0;
-}
-
 QString fixCppTypeName(const QString &name);
 
 #endif // TYPESYSTEM_H
